Imports Dapper
Imports Dapper.Domain
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports QueryTests.Helpers
Imports System.Linq

Namespace QueryTests
  <TestClass>
  Public Class DapperTests
	Private _sw As New Stopwatch
	Private _retrievedObjects As Integer

	<TestMethod>
	Public Sub CanGetADesigner()
	  Using conn = Utils.CreateConnection
		Dim designers = conn.Query(Of DapperDesigner)("select Top 1 * from DapperDesigners")
		Assert.AreEqual(1, designers.Count)
	  End Using
	End Sub

	<TestMethod>
	Public Sub GetAllDesigners()
	  Dim times As New List(Of Long)
	  For i  = 0 To 24
		Using conn = Utils.CreateOpenConnection
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = conn.Query(Of DapperDesigner)("select * from DapperDesigners")
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _retrievedObjects = designers.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "Dapper: GetAllDesignersRawSql")
	  Console.WriteLine($"Latest Retrieved Object Count:{_retrievedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod>
	Public Sub GetAllProjectedDesigners()
	  Dim times As New List(Of Long)
	  For i  = 0 To 24
		Using conn = Utils.CreateOpenConnection
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = conn.Query(Of DapperDesigner)("select  LabelName as Name, id from DapperDesigners")
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _retrievedObjects = designers.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "Dapper: GetAllProjectedDesigners")
	  Console.WriteLine($"Latest Retrieved Object Count:{_retrievedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod>
	Public Sub CanReuseOpenConnection()
	  Dim connStates = New List(Of String)
	  Using conn = Utils.CreateOpenConnection
		For i  = 0 To 2
		  connStates.Add(conn.State.ToString)
		  Dim designers = conn.Query(Of DapperDesigner)("select top 1 * from DapperDesigners")
		Next i
	  End Using
	  CollectionAssert.AreEqual( { "Open", "Open", "Open" }, connStates.ToArray)
	End Sub

	<TestMethod>
	Public Sub GetAllDesignersWithProducts()
	  Dim sql = "select * from DapperDesigners D " & ControlChars.CrLf & "                LEFT OUTER JOIN Products P" & ControlChars.CrLf & "                ON P.DapperDesignerId = D.Id"
	  Dim times As New List(Of Long)
	  Dim designers As IEnumerable(Of DapperDesigner) = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using conn = Utils.CreateOpenConnection
		  _sw.Reset()
		  _sw.Start()
		   designers= conn.Query(Of DapperDesigner, Product,DapperDesigner)(sql, Function(designer, product)
			   designer.Products.Add(product)
			   Return designer
		   End Function)

		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _retrievedObjects = designers.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "Dapper: GetAllDesignersWithProducts")
	  Console.WriteLine($"Latest Retrieved Object Count:{_retrievedObjects}")
	  Assert.AreNotEqual(0,designers.Count)
	End Sub

	<TestMethod>
	Public Sub GetAllDesignersWithContactInfo()
	  Dim sql = "select * from DapperDesigners D " & ControlChars.CrLf & "                LEFT OUTER JOIN ContactInfoes C" & ControlChars.CrLf & "                ON C.Id = D.Id"
	  Dim times As New List(Of Long)
	  Dim designers As IEnumerable(Of DapperDesigner) = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using conn = Utils.CreateOpenConnection
		  _sw.Reset()
		  _sw.Start()
		  designers = conn.Query(Of DapperDesigner, ContactInfo, DapperDesigner)(sql, Function(designer, contact)
			  designer.ContactInfo= contact
			  Return designer
		  End Function)

		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _retrievedObjects = designers.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "Dapper: GetAllDesignersWithContactInfo")
	  Console.WriteLine($"Latest Retrieved Object Count:{_retrievedObjects}")
	  Console.WriteLine($"Proof of Contact: {designers.FirstOrDefault.ContactInfo.Twitter}")
	  Assert.IsNotNull(designers.FirstOrDefault.ContactInfo)
	End Sub

	<TestMethod>
	Public Sub GetAllDesignersWithClients()
	  Dim sql = "SELECT D.Id, d.LabelName,C.Id, C.Name" & ControlChars.CrLf & "                  FROM DapperDesigners D  " & ControlChars.CrLf & "                  LEFT OUTER JOIN DapperDesignerClients dc on (D.Id = dc.DapperDesigner_Id)" & ControlChars.CrLf & "                  LEFT OUTER JOIN Clients C on (dc.Client_Id = C.Id);"
	  Dim times As New List(Of Long)
	  Dim designers As IEnumerable(Of DapperDesigner) = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using conn = Utils.CreateOpenConnection
		  _sw.Reset()
		  _sw.Start()
		  designers = conn.Query(Of DapperDesigner, Client, DapperDesigner) (sql,Function(designer, client)
			  designer.Clients.Add(client)
			  Return designer
		  End Function)

		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _retrievedObjects = designers.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "Dapper: GetAllDesignersWithClients")
	  Console.WriteLine($"Latest Retrieved Object Count:{_retrievedObjects}")
	  Assert.AreNotEqual(0, designers.Select(Function(d) d.Clients).Count)
	End Sub

	<TestMethod>
	Public Sub CanGetAProjectedDesigner()
	  Using conn = Utils.CreateOpenConnection
		Dim designers = conn.Query(Of DapperDesigner)("select top 1 * from DapperDesigners")
		Dim miniDesigners = conn.Query(Of MiniDesigner)("select top 1 LabelName as Name,id from DapperDesigners")
		Assert.AreEqual(designers.First.LabelName,miniDesigners.First.Name)
	  End Using
	End Sub

	<TestMethod>
	Public Sub GetFilteredDesigners()
	End Sub

	<TestMethod>
	Public Sub GetFilteredAndOrderedDesigners()
	End Sub
  End Class
End Namespace
