Imports Dapper.Domain
Imports DapperDesigners.Data.EF6
Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports QueryTests.Helpers
Imports System.Data.Common
Imports System.Data.Entity
Imports System.Data.SqlClient
Imports System.Linq

Namespace QueryTests.vb
  <TestClass>
  Public Class EFTests
	Private _sw As New Stopwatch
	Private _trackedObjects As Integer

	Public Sub New()
	  'warm up EF
	  Using context = New DapperDesignerContext
		context.Designers.Find(1)
		context.Clients.Find(1)
	  End Using
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,Track")>
	Public Sub GetAllDesigners_Tracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesigners")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetAllDesigners_AsNoTracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesigners")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,NoTrack")>
	Public Sub GetAllDesigners_ViaRawSqlNotTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "select * from DapperDesigners"

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.SqlQuery(sql).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersRawSql")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,Track")>
	Public Sub GetAllDesigners_ViaRawSqlTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "select * from DapperDesigners"
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.SqlQuery(sql).ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersRawSql")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetAllDesignersWithProducts_AsNoTracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.Products).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithProductsAsNoTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")
	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,NoTrack")>
	Public Sub GetAllDesignersWithProducts_ViaRawSqlNotTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "select * from DapperDesigners D " & ControlChars.CrLf & "                LEFT OUTER JOIN Products P" & ControlChars.CrLf & "                ON P.DapperDesignerId = D.Id"
	  Dim designers = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  designers = context.Designers.SqlQuery(sql).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithProductsViaRawSqlNotTracked")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.AreNotEqual(0, designers.Select(Function(d) d.Products).Count)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,Track")>
	Public Sub GetAllDesignersWithContact_Tracking()
	  Dim times As New List(Of Long)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.ContactInfo).ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)

	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithContactTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetAllDesignersWithContact_AsNoTracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.ContactInfo).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithContactAsNoTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")
	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,NoTrack")>
	Public Sub GetAllDesignersWithContact_ViaRawSqlNotTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "select * from DapperDesigners D " & ControlChars.CrLf & "                LEFT OUTER JOIN ContactInfoes C" & ControlChars.CrLf & "                ON C.Id = D.Id"
	  Dim designers = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  designers = context.Designers.SqlQuery(sql).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithContact_ViaRawSqlNotTracked")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.AreNotEqual(0, designers.Select(Function(d) d.Clients).Count)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,NoTrack")>
	Public Sub GetAllDesignersWithClients_ViaRawSqlNotTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "SELECT D.*,C.*" & ControlChars.CrLf & "                  FROM DapperDesigners D  " & ControlChars.CrLf & "                  LEFT OUTER JOIN DapperDesignerClients dc on (D.Id = dc.DapperDesigner_Id)" & ControlChars.CrLf & "                  LEFT OUTER JOIN Clients C on (dc.Client_Id = C.Id);"
	  Dim designers = New List(Of DapperDesigner)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  designers = context.Designers.SqlQuery(sql).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithClientsViaRawSqlNotTracked")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.AreNotEqual(0, designers.Select(Function(d) d.Clients).Count)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetAllDesignersWithClients_AsNoTracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.Clients).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithClientsAsNoTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")
	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,Track")>
	Public Sub GetAllDesignersWithContactsAndClients_Tracking()
	  Dim times As New List(Of Long)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.ContactInfo).Include(Function(d) d.Clients).ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithContactsAndClientsTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")
	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetAllDesignersWithContactsAndClients_AsNoTracking()
	  Dim times As New List(Of Long)

	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  Dim designers = context.Designers.Include(Function(d) d.ContactInfo).Include(Function(d) d.Clients).AsNoTracking.ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)

		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetAllDesignersWithContactsAndClientsAsNoTracking")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")
	  Assert.IsTrue(True)
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,NoTrack")>
	Public Sub GetProjectedDesigners()
	  Using context = New DapperDesignerContext
		Dim miniDesigner = context.Designers.AsNoTracking.Select(Function(d) New MiniDesigner With {.Id = d.Id, .Name = d.LabelName, .FoundedBy = d.Founder}).FirstOrDefault
		 Assert.AreNotEqual(" ", miniDesigner.Name)
	  End Using
	End Sub

	<TestMethod, TestCategory("EF"), TestCategory("EF,RawSQL,NoTrack")>
	Public Sub GetProjectedDesigners_ViaRawSqlNotTracked()
	  Dim times As New List(Of Long)
	  Dim sql = "select LabelName as Name,id from DapperDesigners"
	  Dim designers = New List(Of MiniDesigner)
	  For i  = 0 To 24
		Using context = New DapperDesignerContext
		  _sw.Reset()
		  _sw.Start()
		  designers = context.Database.SqlQuery(Of MiniDesigner)(sql).ToList
		  _sw.Stop()
		  times.Add(_sw.ElapsedMilliseconds)
		  _trackedObjects = context.ChangeTracker.Entries.Count
		End Using
	  Next i
	  Dim analyzer = New TimeAnalyzer(times)
	  Utils.Output(times, analyzer, "EF: GetProjectedDesigners_ViaRawSqlNotTracked")
	  Console.WriteLine($"Latest Tracked Objects:{_trackedObjects}")

	  Assert.AreNotEqual(0, designers.Count)
	End Sub

	<TestMethod>
	Public Sub CanReuseOpenConnection()
	  Dim conn As SqlConnection = Utils.CreateOpenConnection
	  Dim connStates = New List(Of String)
	  For i  = 0 To 2
		connStates.Add(conn.State.ToString)
		Using context = New DapperDesignerContext(conn, contextOwnsConnection:= False)
		  Dim designers = context.Designers.SqlQuery("select top 1 from DapperDesigners")
		End Using
	  Next i
	  CollectionAssert.AreEqual( { "Open", "Open", "Open" }, connStates.ToArray)
	End Sub
  End Class
End Namespace
